//#include <qedit.h>
#include <vector>

//#include <iProxyTrans.h>
//#include <ProxyTransuids.h>

//#include "filters.h"



#include <stdio.h>
#include <math.h>
#include <string.h>
#include <dxutil.h>  
#include <streams.h> 
#include "cv.h"      // include core library interface
#include "highgui.h" // include GUI library interface

IPin *GetPin(IBaseFilter *pFilter, PIN_DIRECTION PinDir);

#if !defined IMAGEPROCESSOR
#define IMAGEPROCESSOR

class ImageProcessor {
 
    IplImage* img; // Declare IPL/OpenCV image pointer
	IplImage* tmp;
 
 
  public:
    
    ImageProcessor(CString filename, bool display=true) {
    
      img = cvvLoadImage(filename); // load image
	  tmp= cvCloneImage(img);      //copia di lavoro
 
      if (display) {
 
        // create a window
        cvvNamedWindow("Immagine Originale", 1);
 
        // display the image on window
        cvvShowImage("Immagine Originale", img);  
      }
    }
	void reset(){
	img = cvCloneImage(tmp);
	}
    void display() {
	  
      cvvNamedWindow( "Immagine Risultante", 1 );  
      cvvShowImage( "Immagine Risultante", img );  
    }

    void execute();
    
    void execute2();
	
	void execute3();

    ~ImageProcessor() {
      cvReleaseImage( &img );
    }
};


extern ImageProcessor *proc;
#endif

#if !defined SEQUENCEPROCESSOR
#define SEQUENCEPROCESSOR

class SequenceProcessor {
 
    IplImage* img; // Declare IPL/OpenCV image pointer
    IGraphBuilder *pGraph;
    IMediaControl *pMediaControl;
    IMediaEvent *pEvent;

	bool createFilterGraph(CString filename) {

      WCHAR	*mediaFile= new WCHAR[filename.GetLength()+1];
      MultiByteToWideChar(CP_ACP, 0, filename, -1, mediaFile, filename.GetLength()+1);

	  // Create a source filter
	  IBaseFilter*	pSource= NULL;
	  if(FAILED(pGraph->AddSourceFilter(mediaFile,0,&pSource)))
	  {
        ::MessageBox( NULL, "Unable to create source filter", "Error", MB_OK | MB_ICONINFORMATION );
        return 0;
	  }

      IPin* pSourceOut= GetPin(pSource, PINDIR_OUTPUT);
      if (!pSourceOut) {

        ::MessageBox( NULL, "Unable to obtain source pin", "Error", MB_OK | MB_ICONINFORMATION );
        return 0;
      }


      // Create an AVI splitter filter
      IBaseFilter* pAVISplitter = NULL;
      if(FAILED(CoCreateInstance(CLSID_AviSplitter, NULL, CLSCTX_INPROC_SERVER,
                IID_IBaseFilter, (void**)&pAVISplitter)) || !pAVISplitter)
      {
        ::MessageBox( NULL, "Unable to create AVI splitter", "Error", MB_OK | MB_ICONINFORMATION );
        return 0;
      }

      IPin* pAVIsIn= GetPin(pAVISplitter, PINDIR_INPUT);
      if (!pAVIsIn) {

        ::MessageBox( NULL, "Unable to obtain input splitter pin", "Error", MB_OK | MB_ICONINFORMATION );
        return 0;
      }

      // Connect the source and the splitter
      if(FAILED(pGraph->AddFilter( pAVISplitter, L"Splitter")) ||
         FAILED(pGraph->Connect(pSourceOut, pAVIsIn)) )           
      {
        ::MessageBox( NULL, "Unable to connect AVI splitter filter", "Error", MB_OK | MB_ICONINFORMATION );
        return 0;
      }

      // Create an AVI decoder filter
      IBaseFilter* pAVIDec = NULL;
      if(FAILED(CoCreateInstance(CLSID_AVIDec, NULL, CLSCTX_INPROC_SERVER,
                IID_IBaseFilter, (void**)&pAVIDec)) || !pAVIDec)
      {
        ::MessageBox( NULL, "Unable to create AVI decoder", "Error", MB_OK | MB_ICONINFORMATION );
        return 0;
      }

      IPin* pAVIsOut= GetPin(pAVISplitter, PINDIR_OUTPUT);
      if (!pAVIsOut) {

        ::MessageBox( NULL, "Unable to obtain output splitter pin", "Error", MB_OK | MB_ICONINFORMATION );
        return 0;
      }

      IPin* pAVIDecIn= GetPin(pAVIDec, PINDIR_INPUT);
      if (!pAVIDecIn) {

        ::MessageBox( NULL, "Unable to obtain decoder input pin", "Error", MB_OK | MB_ICONINFORMATION );
        return 0;
      }

      // Connect the splitter and the decoder
      if(FAILED(pGraph->AddFilter( pAVIDec, L"Decoder")) ||
         FAILED(pGraph->Connect(pAVIsOut, pAVIDecIn)) )           
      {
        ::MessageBox( NULL, "Unable to connect AVI decoder filter", "Error", MB_OK | MB_ICONINFORMATION );
        return 0;
      }

      // Render the stream from the decoder
      IPin* pAVIDecOut= GetPin(pAVIDec, PINDIR_OUTPUT);
      if (!pAVIDecOut) {

        ::MessageBox( NULL, "Unable to obtain decoder output pin", "Error", MB_OK | MB_ICONINFORMATION );
        return 0;
      }

      if(FAILED(pGraph->Render( pAVIDecOut ))) 
      {
        ::MessageBox( NULL, "Unable to connect to renderer", "Error", MB_OK | MB_ICONINFORMATION );
        return 0;
      }

      SAFE_RELEASE(pAVIDecIn);
      SAFE_RELEASE(pAVIDecOut);
      SAFE_RELEASE(pAVIDec);
      SAFE_RELEASE(pAVIsOut);
      SAFE_RELEASE(pAVIsIn);
      SAFE_RELEASE(pAVISplitter);
      SAFE_RELEASE(pSourceOut);
      SAFE_RELEASE(pSource);

      return 1;
    }

  public:
    
    SequenceProcessor(CString filename, bool display=true) {
    
      CoInitialize(NULL);

      pGraph= NULL;
      pMediaControl= NULL;
      pEvent= NULL;

      if (!FAILED(
             CoCreateInstance(CLSID_FilterGraph, NULL, CLSCTX_INPROC, 
                              IID_IGraphBuilder, (void **)&pGraph))) {

        pGraph->QueryInterface(IID_IMediaControl, (void **)&pMediaControl);
        pGraph->QueryInterface(IID_IMediaEvent, (void **)&pEvent);

        if (createFilterGraph(filename) && display) {

          pMediaControl->Run();

          // Wait for completion. 
          long evCode;
          pEvent->WaitForCompletion(INFINITE, &evCode);
        }
      }
    }

    std::vector<CString> enumFilters() {

      IEnumFilters *pEnum = NULL;
      IBaseFilter *pFilter;
      ULONG cFetched;
      std::vector<CString> names;

      pGraph->EnumFilters(&pEnum);
      while(pEnum->Next(1, &pFilter, &cFetched) == S_OK)
      {
        FILTER_INFO FilterInfo;
        char szName[256];
        CString fname;

        pFilter->QueryFilterInfo(&FilterInfo);
        WideCharToMultiByte(CP_ACP, 0, FilterInfo.achName, -1, szName, 256, 0, 0);
        fname= szName;
        names.push_back(fname);

        SAFE_RELEASE(FilterInfo.pGraph);
        SAFE_RELEASE(pFilter);
      }

      SAFE_RELEASE(pEnum);

      return names;
    }

    ~SequenceProcessor() {

      SAFE_RELEASE(pMediaControl);
      SAFE_RELEASE(pEvent);
      SAFE_RELEASE(pGraph);

      CoUninitialize();
    }
};

extern SequenceProcessor *procseq;

#endif
